#include "Configuration.h"
#include "World.h"

#include <iostream>

using namespace std;
char printStr[30];

void printIterations()
{
	int font=(int)GLUT_BITMAP_9_BY_15;
	
	glColor3f(1.0f, 1.0f, 1.0f);
	glPushMatrix();
	glLoadIdentity();

	//switch to ortho view for text draw
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();//saves old matrix
	glLoadIdentity();
	gluOrtho2D(0, SCREEN_WIDTH, 0, SCREEN_HEIGHT);
	glScalef(1, -1, 1);//invert screen coords (y+ is down)
	glTranslatef(0, -SCREEN_HEIGHT, 0);
	glMatrixMode(GL_MODELVIEW);

	char* currChar;

	//print iteration number
	glRasterPos2f(20, 40);//set x and y start pos
	char iterationStr[30];
	sprintf(iterationStr, "Iterations : %d", World::getInstance()->getIterations());
	for(currChar = iterationStr; *currChar != '\0'; currChar++)
	{
		glutBitmapCharacter((void *)font, *currChar);
	}

	glPopMatrix();
	
	//switch matrix back to projection
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();//get old settings
	glMatrixMode(GL_MODELVIEW);
}

/**
*	Called every frame to redraw images to the screen
*/
void Render()
{
	glClear(GL_COLOR_BUFFER_BIT); //Clear screen
	World::getInstance()->Render();
	printIterations();
	glFlush();
}

/**
*	Calls the World object's update function to recalculate object positions.
*	Renders the new World, then sets a timer for 16ms to recall this method.
*/
void DispatchTimer(int ID) 
{
	World::getInstance()->Update();
	Render();
	glutTimerFunc((int)(1000.0f/60.0f), DispatchTimer, 0); //Call this function again in 16 ms
}

/**
*	Called whenever the user presses a key on the keyboard.
*	Handles the escape key to exit, and wasd keys to alter direction of gravity.
*/
void DispatchKeyboard(unsigned char key, int x, int y)
{
	//Esc key closes program
	if(key == 27)
		exit(0);
	else if(key == 'w')
		World::getInstance()->setGravity(Vec2(0.0f, -0.2f));
	else if(key == 'a')
		World::getInstance()->setGravity(Vec2(-0.2f, 0.0f));
	else if(key == 's')
		World::getInstance()->setGravity(Vec2(0.0f, 0.2f));
	else if(key == 'd')
		World::getInstance()->setGravity(Vec2(0.2f, 0.0f));
	else if(key == '-' || key == '_')
		World::getInstance()->decreaseIterations();
	else if(key == '+' || key == '=')
		World::getInstance()->increaseIterations();

}

/**
*	Adds a number of RigidBodies to the world. Squares and Triangles are added which can be affected by
*	collisions. Obstacles are added which will not move when collided with.
*/
void InitWorld() 
{
	//Add squares
	for(int x = 20; x < SCREEN_WIDTH; x += 140) 
	{
		for(int y = 50; y < SCREEN_HEIGHT - 50; y += 100)
		{
			RigidBody* rb = new RigidBody(SQUARE);
			rb->finishRigidBody(x, y);
			World::getInstance()->AddBody(rb);
		}
	}

	//Add triangles
	for(int x = 155; x < SCREEN_WIDTH; x += 150)
	{
		for(int y = 100; y < SCREEN_HEIGHT / 2; y += 150)
		{
			RigidBody* rb = new RigidBody(TRIANGLE);
			rb->finishRigidBody(x, y);
			World::getInstance()->AddBody(rb);
		}
	}

	//Add obstacles
	RigidBody* rb1 = new RigidBody(OBSTACLE);
	rb1->finishRigidBody(200, 300);
	World::getInstance()->AddObstacle(rb1);

	RigidBody* rb2 = new RigidBody(OBSTACLE);
	rb2->finishRigidBody(100, 120);
	World::getInstance()->AddObstacle(rb2);

	RigidBody* rb3 = new RigidBody(OBSTACLE);
	rb3->finishRigidBody(510, 450);
	World::getInstance()->AddObstacle(rb3);

	RigidBody* rb4 = new RigidBody(OBSTACLE);
	rb4->finishRigidBody(515, 150);
	World::getInstance()->AddObstacle(rb4);

}

/**
*	Initializes OpenGL and creates application window
*/
void InitGL() {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	glOrtho(0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, -1, 1);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

/**
*	Creates the window, sets the display mode, and starts the timer to re-render the screen.
*	Also creates listener for key events, creates the World, and adds RigidBodies to it.
*/
int main( int argc, char **argv ) 
{
	glutInit(&argc, argv);

	glutInitDisplayMode(GLUT_DEPTH | GLUT_SINGLE | GLUT_RGBA);
	glutInitWindowPosition(0, 0);
	glutInitWindowSize(SCREEN_WIDTH, SCREEN_HEIGHT);
	glutCreateWindow("RigidBody Demo");

	glutDisplayFunc(Render);
	glutTimerFunc((int)(100.0f/60.0f), DispatchTimer, 0);
	glutKeyboardFunc(DispatchKeyboard);

	InitGL();
	InitWorld();
	glutMainLoop();

	World::getInstance()->Destroy();

	return 0;
}
