/**
*	Name:		Elizabeth Labelle
*	ID:			17417
*	Project:	2D Verlet Rigid Body World
*	Date:		March 2010
*
*	RigidBody struct.
*	A RigidBody is comprised of vertices (points), and edges that connect them.
*	World Bodies can have any number of points and edges, to create any shape.
*	Each RigidBody has a center, a bounding box for collision detection, and an array
*	of vertices and edges.
*/

#ifndef RIGIDBODY_H
#define RIGIDBODY_H

#include "Edge.h" //includes Vertex.h

/** enum for RigidBody types **/
enum Shape {OBSTACLE, //Rectangle which is not affected by physics, holds other shapes
			TRIANGLE,
			SQUARE};

struct RigidBody
{
	/** 2D point describing the center of the RigidBody **/
	Vec2 center;

	/** Creates a square bounding box around the rigidbody **/
	int minX, minY, maxX, maxY;

	/** The maximum number of vertices in the RigidBody. Based on the shape. **/
	int maxVerts;
	/** The maximum number of edges in the RigidBody. Based on the shape. **/
	int maxEdges;
	/** The current number of vertices in the RigidBody. Must not exceed maxVerts **/
	int vertexCount;
	/** The current number of edges in the RigidBody. Must not exceed maxEdges **/
	int edgeCount;
	/** Enum describing the shape of the RigidBody **/
	Shape shape;

	//The number of edges and vertices changes depending on the type of shape
	//We will initialize these arrays in the constructor
	Vertex* vertexArray[4];
	Edge*  edgeArray[6];

	/**
	*	Constructor
	*/
	RigidBody(Shape s);
	~RigidBody();

	void finishRigidBody(int x, int y);

	void AddEdge(Edge* e);
	void AddVertex(Vertex* v);
	void ProjectToAxis(Vec2& axis, float& min, float& max);
	void CalculateCenter();
};

#endif RIGIDBODY_H