/**
*	Name:		Elizabeth Labelle
*	ID:			17417
*	Project:	2D Verlet Rigid Body World
*	Date:		March 2010
*
*	Description:	Vec2.h is used to describe/create a simple 2D Vector class.
*					A Vec2 has an x and y value. Vec2s can be added, subtracted, multiplied, divided,
*					checked for equality, and normalized.
*					Basic operators (+,-,*,/, +=, -=, etc) are overloaded for the Vec2 type.
*/

#ifndef VEC2_H
#define VEC2_H

#include <math.h>

struct Vec2 {
	float x;
	float y;

	/**
	*	Constructor
	*/
	Vec2(float pX = 0.0f, float pY = 0.0f) : x(pX), y(pY)
	{}

	/**
	*	Destructor
	*/
	~Vec2()
	{}

	//Overload operators for Vec2 type

	Vec2 operator*(float number)
	{ 
		return Vec2(x*number, y*number);
	}

	Vec2 operator/(float number)
	{
		return Vec2(x/number, y/number);
	}

	Vec2 operator+(Vec2 B)
	{
		return Vec2(x + B.x, y + B.y);
	}

	Vec2 operator-(Vec2 B)
	{ 
		return Vec2(x - B.x, y - B.y);
	}

	Vec2 operator-()
	{
		return Vec2(-x,-y);
	}

	//Overload operators for non-Vec2 type

	float operator*(Vec2 B)
	{ 
		return x*B.x + y*B.y;
	}

	void operator=(Vec2 B)
	{
		x = B.x;
		y = B.y;
	}
	
	void operator+=(Vec2 B)
	{
		x += B.x;
		y += B.y;
	}
	
	void operator-=(Vec2 B)
	{ 
		x -= B.x;
		y -= B.y;
	}

	void operator*=(float number)
	{
		x *= number;
		y *= number;
	}
	
	void operator/=(float number)
	{
		x /= number;
		y /= number;
	}
	
	bool operator==(Vec2 B)
	{
		return x == B.x && y == B.y;
	}

	bool operator!=(Vec2 B)
	{
		return x != B.x || y != B.y;
	}

	/**
	*	Returns the length of the vector
	*/
	float length()
	{
		return sqrtf(x*x + y*y);
	}

	/**
	*	Returns the length of the vector squared
	*/
	float lengthSq()
	{
		return x*x + y*y;
	}

	/**
	*	Returns a normalized version of the vector with values from 0-1
	*/
	void normalize()
	{
		float Len = 1.0/sqrtf(x*x + y*y);

		x = x * Len;
		y = y * Len;
	}
};

#endif VEC2_H