/**
*	Name:		Elizabeth Labelle
*	ID:			17417
*	Project:	2D Verlet Rigid Body World
*	Date:		March 2010
*
*	Description:	The World class describes the world environment in the program. It keeps track of
*					all RigidBodies in the environment and forces like gravity which affect them. It
*					runs an update loop which calculates forces, position, and collisions for each RigidBody
*					and moves the RigidBodies accordingly.
*/

#ifndef World_H
#define World_H

#ifdef _WIN32
#include <windows.h>
#endif

#include "Configuration.h"
#include "CurrentImpact.h"
#include "RigidBody.h" //includes Edge.h and Vertex.h

class World
{
private:
	//made private because it is a singleton
	World();
	/** Singleton instance of the World class*/
	static World* instance;

	/** Holds the current direction of gravity in the World **/
	Vec2 gravity;
	/** Holds data about the last impact detected in the World so it can be handled. **/
	CurrentImpact currentImpact;

	/** Number of RigidBodies of type OBSTACLE in the ObstacleArray **/
	int totalObstacles;
	/** Number of RigidBodies of any type in the RigidBodyArray. **/
	int totalBodies;
	/** Number of vertices that make up the RigidBodies in the RigidBodyArray **/
	int totalVertices;
	/** Number of edges that make up the RigidBodies in the RigidBodyArray **/
	int totalEdges;

	/** Holds RigidBodies of type OBSTACLE in the World. These do not move when collided with. **/
	RigidBody* ObstacleArray[MAX_OBSTACLES];
	/** Holds RigidBodies of any type that are affected by collsions in the World. **/
	RigidBody* RigidBodyArray[MAX_OBJECTS];
	/** The amount of time between ticks. **/
	float tick;
	/**
	*	The number of times to run the edge correction step each update. Increasing the number
	*	reduces the amount of edge length deformation caused by collisions moving seperate vertices,
	*	but increases the amount of computation needed each update.
	*/
	int iterations;

	//Update Methods
	void updateForces();
	void updateVerlet();
	void updateEdges();
	bool didCollide(RigidBody* pb1, RigidBody* pb2);
	bool overlapping(RigidBody* pb1, RigidBody* pb2);
	void handleCollision();
	void processAllCollisions();
	float getDistance(float minA, float maxA, float minB, float maxB);

public:
	static World* getInstance();

	void Destroy();

	void Update();
	void Render();
	void AddBody(RigidBody* Body);
	void AddObstacle(RigidBody* obstacle);
	void setGravity(Vec2 newGravity);
	int getIterations();
	void increaseIterations();
	void decreaseIterations();
};

#endif World_H